<?php
/**
 * SOTESHOP/stUpdate
 *
 * Ten plik należy do aplikacji stUpdate opartej na licencji (Open License SOTE) Otwarta Licencja SOTE. 
 * Nie zmieniaj tego pliku, jeśli chcesz korzystać z automatycznych aktualizacji oprogramowania. 
 * Jeśli chcesz wprowadzać swoje modyfikacje do programu, zapoznaj się z dokumentacją, jak zmieniać 
 * oprogramowanie bez zmiany kodu bazowego http://www.sote.pl/modifications
 *
 * @package     stUpdate
 * @subpackage  actions
 * @copyright   SOTE (www.sote.pl)
 * @license     http://www.sote.pl/license/open (Open License SOTE) Otwarta Licencja SOTE
 * @version     $Id: actions.class.php 8843 2010-10-20 10:08:06Z piotr $
 * @author      Piotr Halas <piotr.halas@sote.pl>
 * @author      Michal Prochowski <michal.prochowski@sote.pl>
 */

/**
 * Klasa stSetupActions
 *
 * @package     stUpdate
 * @subpackage  actions
 */
class stSetupActions extends sfActions
{
	/**
	 * Wyświetlanie informacji ogólnych
	 */
	public function executeIndex()
	{
		$request = $this->getRequest();
		if ($request->hasParameter('language') && $request->getParameter('language') == 'en') $this->getUser()->setCulture('en_US');
		else $this->getUser()->setCulture('pl_PL');
			
		$pakeweb = new stPakeWeb();
		$pakeweb->run('pear-update-config');

		$this->error = true;

		$this->license = $request->getParameter('license', null);

		if ($request->getMethod() == sfRequest::POST)
		{
			if ($request->hasParameter('license') && !empty($this->license))
			{
				$stLicense = new stLicense($this->license);
				if ($stLicense->check())
				{
					$this->getUser()->setAttribute('licenseType', $stLicense->getType(), 'soteshop/stSetup');
					
					$file = sfConfig::get('sf_root_dir').DIRECTORY_SEPARATOR."install".DIRECTORY_SEPARATOR."db".DIRECTORY_SEPARATOR.".license.reg";
					if (file_exists($file)) @unlink($file);
					file_put_contents($file, $this->license);
					
					$this->redirect('stSetup/license');
				} else {
					$this->errorMsg = 'Numer licencji jest nieprawidłowy.';
					return sfView::SUCCESS;
				}
			} else {
				$this->errorMsg = 'Proszę podać numer licencji.';
				return sfView::SUCCESS;
			}
		}

		$this->error = false;
	}

	/**
	 * Wyświetlanie licencji
	 */
	public function executeLicense()
	{
		switch($this->getUser()->getAttribute('licenseType', null, 'soteshop/stSetup'))
		{
			case stLicense::LICENSE_TYPE_COMMERCIAL:
				$this->partial = 'license_commercial';
				break;
			case stLicense::LICENSE_TYPE_OPEN:
				$this->partial = 'license_open';
				break;
			default:
				$this->partial = 'license_commercial';
				break;
		}

		if ($this->getUser()->getCulture() == 'en_US') $this->partial.= '_en';

		$this->notAccepted = false;

		if ($this->getRequest()->getMethod() == sfRequest::POST)
		{
			if ($this->getRequestParameter('accept')) $this->redirect('stSetup/require');
			$this->notAccepted = true;
		}
	}

	/**
	 * Wyświetlanie wymagań
	 */
	public function executeRequire()
	{
		$this->tests = new stSetupRequirements();
		$this->testsPassed = $this->tests->testAll();
		$this->testsStatus = $this->tests->getTest();
		$this->hasWarning = false;
	}

	/**
	 * Wyświetlanie konfiguracji bazy danych
	 */
	public function executeDatabase()
	{
		$this->dbUsername = $this->getRequestParameter('username');
		$this->dbPassword = $this->getRequestParameter('password');
		$this->dbHost = $this->getRequestParameter('host');
		$this->dbDatabase = $this->getRequestParameter('database');

		$this->dbError = true;
		$this->dbErrorMsg = '';

		if ($this->getRequest()->getMethod() == sfRequest::POST)
		{
			$con = @mysql_connect($this->dbHost,$this->dbUsername, $this->dbPassword);
			if ($con && mysql_select_db($this->dbDatabase,$con))
			{
				$res = mysql_query('SHOW VARIABLES LIKE "collation_database"', $con);
				$data = mysql_fetch_array($res);
				
				if ($data[1]!="utf8_unicode_ci") {
					mysql_free_result($res);
					$this->dbErrorMsg = 'Złe kodowanie bazy danych, proszę zmienić na utf8_unicode_ci.';
					return sfView::SUCCESS;
				}
				
				$res = mysql_query('SHOW TABLES', $con);

				if (!mysql_num_rows($res))
				{
					mysql_free_result($res);
					$setupFile = sfConfig::get('sf_config_dir').DIRECTORY_SEPARATOR.'__st_setup.yml';

					$database = array( 'database' => array (
					                   'username' => $this->dbUsername,
					                   'password' => $this->dbPassword,
					                   'host' => $this->dbHost,
					                   'database' => $this->dbDatabase
					));
					if (!file_put_contents($setupFile,sfYaml::dump($database)))
					{
						$this->dbErrorMsg = 'Nie moge zapisać pliku konfiguracyjnego';
						return sfView::SUCCESS;
					}
					$this->redirect('stSetup/download');
				} else {
					mysql_free_result($res);
					$this->dbErrorMsg = 'Baza danych nie jest pusta.';
					return sfView::SUCCESS;
				}
			} else {
				$this->dbErrorMsg = 'Nie można połączyć się z bazą danych, sprawdz dane i spróbuj ponownie.';
				return sfView::SUCCESS;
			}
		}
		$this->dbError = false;
	}

	/**
	 * Wyświetlanie ściągania pakietów
	 */
	public function executeDownload()
	{
		$dbFile = sfConfig::get('sf_root_dir').DIRECTORY_SEPARATOR."install".DIRECTORY_SEPARATOR."db".DIRECTORY_SEPARATOR.".packages.reg";

		$pear = stPearInfo::getInstance()->getDefaultChannel();
		$soteshopUrl = 'http://'.$pear.'/Chiara_PEAR_Server_REST/r/soteshop/latest.txt';
		$seteshopVersion = $this->getUrlResponse($soteshopUrl);
		$package = array('soteshop' => $seteshopVersion);

		$deps = $this->getDepsFor($package,$pear);
		file_put_contents($dbFile, serialize($deps));
	}

	/**
	 * Wyświetlanie instalacji
	 */
	public function executeInstall() {}

	/**
	 * Wyświetlanie konfiguracji i optymalizacji
	 */
	public function executeConfigure() {

		// włącz fastcache dla nowych instalacji
		$fastCacheEnabled = SF_ROOT_DIR.DIRECTORY_SEPARATOR.'fastcache'.DIRECTORY_SEPARATOR.'db'.DIRECTORY_SEPARATOR.'fast_cache_enabled';
		if (!file_exists($fastCacheEnabled)) touch($fastCacheEnabled);	
	}

	/**
	 * Wyświetlenie informacji końcowych
	 */
	public function executeFinish()
	{
		$hostname = $this->getRequest()->getHost();
			
		$this->frontend = "http://$hostname";
		$this->backend = "http://$hostname/backend.php";
		$this->update = "http://$hostname/update.php";
			
		$file = sfConfig::get('sf_app_dir').DIRECTORY_SEPARATOR.'modules'.DIRECTORY_SEPARATOR.'stSetup'.DIRECTORY_SEPARATOR.'config'.DIRECTORY_SEPARATOR.'security.yml';
		$data = array();
		$data['all']['is_secure'] = 'on';
		file_put_contents($file, sfYaml::dump($data));

		$pakeweb = new stPakeWeb();
		if (!$pakeweb->run('cc')) throw new Exception($pakeweb->error);
	}

	/**
	 * Wyświetlenie rekonfiguracji bazy danych
	 */
	public function executeReconfigure()
	{
		$pakeweb = new stPakeWeb();
		$pakeweb->run('pear-update-config');

		$this->dbUsername = $this->getRequestParameter('username');
		$this->dbPassword = $this->getRequestParameter('password');
		$this->dbHost = $this->getRequestParameter('host');
		$this->dbDatabase = $this->getRequestParameter('database');

		$this->dbError = true;
		$this->dbErrorMsg = '';

		if ($this->getRequest()->getMethod() == sfRequest::POST) {

			$con = @mysql_connect($this->dbHost,$this->dbUsername, $this->dbPassword);
			if ($con && mysql_select_db($this->dbDatabase,$con)) {

				$setupFile = sfConfig::get('sf_config_dir').DIRECTORY_SEPARATOR.'__st_setup.yml';

				$database = array('database'=>array(
                       'username'=>$this->dbUsername,
                       'password'=>$this->dbPassword,
                       'host'=>$this->dbHost,
                       'database'=>$this->dbDatabase
				));
				if (!file_put_contents($setupFile,sfYaml::dump($database))) {
					$this->dbErrorMsg = 'Nie moge zapisać pliku konfiguracyjnego';
					return sfView::SUCCESS;
				}

				$pakeweb->run('setup-update');
				return $this->forward('stSetup','reconfigureFinish');

			} else {
				$this->dbErrorMsg = 'Nie można połączyć się z bazą danych, sprawdz dane i spróbuj ponownie.';
				return sfView::SUCCESS;
			}
		}
		$this->dbError = false;
	}

	/**
	 * Wyświetlenie podsumowania rekonfiguracji bazy danych
	 */
	public function executeReconfigureFinish()
	{
		$this->dbUsername = $this->getRequestParameter('username');
		$this->dbPassword = $this->getRequestParameter('password');
		$this->dbHost = $this->getRequestParameter('host');
		$this->dbDatabase = $this->getRequestParameter('database');
	}

	/**
	 * Ściąganie danych o pakietach
	 *
	 * @param string $url
	 */
	private function getUrlResponse($url)
	{
		$file = sfConfig::get('sf_cache_dir').DIRECTORY_SEPARATOR.'chiara_temporaray_file.tmp';
		if (file_exists($file)) unlink($file);

		$fd = fopen($file, "w");
		$c = curl_init();
		curl_setopt($c, CURLOPT_URL,  $url);
		curl_setopt($c, CURLOPT_FILE, $fd);
		curl_exec($c);
		curl_close($c);
		fclose($fd);
			
		return stFile::read($file);
	}

	/**
	 * Pobireanie zależności pakietów 
	 * @param $packages
	 * @param $channel
	 */
	private function getDepsFor($packages = array(), $channel = '')
	{
		$content = "deps=".serialize($packages);
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, "http://".$channel."/deps.php");
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $content);
		$data = curl_exec($ch);
		curl_close($ch);
		return unserialize($data);
	}
}