<?php
/** 
 * SOTESHOP/stUpdate 
 * 
 * This file is the part of stUpdate application. License: (Open License SOTE) Open License SOTE. 
 * Do not modify this file, system will overwrite it during upgrade.
 * 
 * @package     stUpdate
 * @subpackage  libs
 * @copyright   SOTE (www.sote.pl)
 * @license     http://www.sote.pl/license/open (Open License SOTE) Open License SOTE
 * @version     $Id: stSetupTasks.class.php 8546 2010-10-01 09:02:24Z piotr $
 * @author      Marek Jakubowicz <marek.jakubowicz@sote.pl>
 */

/** 
 * Installation. Progress bar class.
 *
 * @package     stUpdate
 * @subpackage  libs
 */
class stSetupTasks extends stInstallerTasks {


  /** 
   * Step.
   *
   * @param   integer     $step               step number
   * @return  integer     next step number (default $step+1)
   */
  	public function step($step)
	{
		$pakeweb = new stPakeWeb();
		$task=''; $sync=false;

		 
		switch ($step)
		{
			case 0: 
            	$this->msg = "Blokowanie sklepu";  // opis krok do przodu
				break;
			case 1:
				$this->msg = "Czyszczenie plików tymczasowych";  // opis krok do przodu
				$this->clearFastCache();
				if (class_exists('stLock'))
				{
					stLock::lock('frontend');
					stLock::lock('backend');
				}
				// lock shop
                $task='cc';
				break;
			case 2: 
				$this->msg = "Kopiowanie plików";  // opis krok do przodu
				$task='cc'; 
				break;
			case 3: 
				$this->msg = "Konfiguracja instalacji";  // opis krok do przodu
				$task=$this->_sync(); 
				$sync=true; 
				break; 
			case 4: 
				$this->msg = "Tworzenie struktury bazy danych";  // opis krok do przodu
				$task='setup-update'; 
				break;         // database update
			case 5: 
				$this->msg = "Tworzenie modelu bazy danych";  // opis krok do przodu
				$task='installer-schema-diff'; 
				break;
			case 6:
				$this->msg = "Czyszczenie plików tymczasowych";  // opis krok do przodu 
				$task=$this->_installer_build_all(); 
				break;
			case 7: 
				$this->msg = "Wczytywanie danych";  // opis krok do przodu				
				$task='cc'; 
				break;
			case 8:
				$this->msg = "Uruchomienie sklepu";  // opis krok do przodu
				sfLoader::loadPluginConfig(); 
				$task='propel-load-data ../backend'; break;
			case 9:
				if ($this->_installVerification())
				{
				    // unlock shop
					if (class_exists('stLock'))
					{
						stLock::unlock('frontend');
						stLock::unlock('backend');
						$this->cleanHistory();
					}
				}
				break; 
		}

		if (! empty($task))
		{

			if ($pakeweb->run($task))
			{
				if (! empty($pakeweb->content)) $this->log("\n".date('Y-m-d G:i:s')."\n".'symfony '.$task."\n".$pakeweb->content);  // save log
			} else {
				throw new Exception($pakeweb->error);
			}

		}

		// // if applications didn't synchronize properly, repeat step
		if ($sync) {
			$apps_sync=$this->_appsToSync();
			if (sizeof($apps_sync)>0) return $step;
		}
		

        if ($task=='setup-update')
        {
            // rebuild robots.txt file
            $this->robotsTxtUpate();
            // rebuild web/.htaccess file
            $this->_htaccess(true);
        }

		return $step+1;
	}

    public function getTitle()
    {
        sfLoader::loadHelpers('I18N');
        return __("Instalacja aplikacji (Uwaga! Nie zamykaj okna przeglądarki, aż instalacja się nie skończy):", null, 'stInstallerWeb');
    }
    
    public function getFatalMessage()
    {
    	sfLoader::loadHelpers('I18N');
    	sfLoader::loadHelpers('Url');
    	
    	return __("Wystąpił błąd podczas instajacji oprogramowania SOTESHOP 5.0, większość problemtów można rozwiązać korzystając z").
    		' <strong>'.link_to(__('Pordręcznika instalacji'), __('http://www.sote.pl/trac/wiki/doc/soteshop_installation'),array('target'=>'_blank')).'</strong>';
    }
	
	
	public function close() {

        $without_optimization=0;
		parent::close($without_optimization);

		sfLoader::loadHelpers('Tag');
		sfLoader::loadHelpers('Javascript');

		$this->msg.="<script type=\"text/javascript\">document.getElementById('stSetup-install_actions').style.visibility=\"visible\";</script>";

	}
        
    /** 
     * Delete history files after installation.
     */
    protected function cleanHistory()
    {
        $file=sfConfig::get('sf_root_dir').DIRECTORY_SEPARATOR.'install'.DIRECTORY_SEPARATOR.'db'.DIRECTORY_SEPARATOR.'.history.reg';
        if (file_exists($file)) unlink ($file);
    }
    
    /** 
     * Rebuild web/robots.txt
     */
    protected function robotsTxtUpate()
    {
        $from = sfConfig::get('sf_root_dir').DIRECTORY_SEPARATOR.'install'.DIRECTORY_SEPARATOR.'src'.DIRECTORY_SEPARATOR.'stPositioningPlugin'.DIRECTORY_SEPARATOR.
        'stPositioningPlugin'.DIRECTORY_SEPARATOR.'web'.DIRECTORY_SEPARATOR.'robots.txt';
        $to  = sfConfig::get('sf_web_dir').DIRECTORY_SEPARATOR.'robots.txt';
        copy($from,$to);
    }
}