<?php
/** 
 * SOTESHOP/stUpdate 
 * 
 * This file is the part of stUpdate application. License: (Open License SOTE) Open License SOTE. 
 * Do not modify this file, system will overwrite it during upgrade.
 * 
 * @package     stUpdate
 * @subpackage  libs
 * @copyright   SOTE (www.sote.pl)
 * @license     http://www.sote.pl/license/open (Open License SOTE) Open License SOTE
 * @version     $Id: stInstallerTasks.class.php 8768 2010-10-13 12:43:27Z piotr $
 * @author      Marek Jakubowicz <marek.jakubowicz@sote.pl>
 */
 

/** 
 * Token keys.
 */
require_once (sfConfig::get('sf_app_lib_dir').DIRECTORY_SEPARATOR.'stToken.php');

/** 
 * Logs
 */
if (! defined('ST_INSTALLER_LOG_PAGE')) define("ST_INSTALLER_LOG_PAGE",sfConfig::get('sf_log_dir').DIRECTORY_SEPARATOR.'webinstaller.log');  

/** 
 * Temporary list of synchronized applications.
 */
define ("ST_APPSTOSYNC_FILE",sfConfig::get('sf_root_dir').DIRECTORY_SEPARATOR.'install'.DIRECTORY_SEPARATOR.'db'.DIRECTORY_SEPARATOR.'.appstosync.reg');

/** 
 * Temporary list of applications installed before synchonization.
 */
define ("ST_REGSYNC_PRE_INSTALL_FILE",sfConfig::get('sf_root_dir').DIRECTORY_SEPARATOR.'install'.DIRECTORY_SEPARATOR.'db'.DIRECTORY_SEPARATOR.'.regsync.reg');

sfLoader::loadHelpers('Helper');
use_helper('I18N','Url', 'Tag'); 
use_helper('stProgressBar','Partial');

/** 
 * Installer WWW. Progress bar steps.
 *
 * @package     stUpdate
 * @subpackage  libs
 */
class stInstallerTasks {
    
    /** 
     * Message (step)
     * @var string
     */
    public $msg = '';
    
    /** 
     * Step
     *
     * @param   integer     $step               count step
     * @return  integer     new step number (default should be $step+1)
     */
    public function step($step)
    {   
        $pakeweb = new stPakeWeb();
        $task=''; $sync=false;      
          
        // if (! stRegisterUpdate::check()) return $step+1;                           
           
        switch ($step)
        {                 
            case 0:     
            	$this->msg = "Blokowanie sklepu";  // opis krok do przodu
                if (stLockUpdate::isLocked())  $locked_before_run=true; else $locked_before_run=false;
                stLockUpdate::lock();                
                $this->_secureStSetup();
                // save apps if instalation wasn't stoped wrongly   
                if (! $locked_before_run) 
                {
                    $this->_saveAppsToSync(); 
                    $this->_saveRegSync();
                }
                break;  
            case 1:         
            	$this->msg = "Czyszczenie plików tymczasowych";  // opis krok do przodu
            	$this->clearFastCache();   
                if (class_exists('stLock'))
                {
                    // lock shop
                    stLock::lock('frontend');
                    stLock::lock('backend');
                }   

                break; 
            case 2: 
            	$this->msg = "Kopiowanie plików";  // opis krok do przodu
            	$task='cc'; 
            	break;                                   
            case 3: 
            	$this->msg = "Konfiguracja instalacji";  // opis krok do przodu
            	$task=$this->_sync(); 
            	$sync=true; 
            	break;
            case 4: 
            	$this->msg = "Tworzenie struktury bazy danych";  // opis krok do przodu
            	$task='setup-update'; 
            	break;                                        // database update
            case 5: 
            	$this->msg = "Tworzenie modelu bazy danych";  // opis krok do przodu
            	$task='installer-schema-diff'; 
	            break;
            case 6: 
            	$this->msg = "Czyszczenie plików tymczasowych";  // opis krok do przodu
            	$task=$this->_installer_build_all(); 
            	break;
            case 7: 
            	$this->msg = "Wczytywanie danych";  // opis krok do przodu
            	$task='cc'; 
            	break;
            case 8: $this->_propelLoadData($this->_loadAppsToSync('added')); break;     // load fixtures only for new applications   
        }                 
          
        if (! empty($task)) 
        {              
            
            if ($pakeweb->run($task))
            {
                if (! empty($pakeweb->content)) $this->log("\n".date('Y-m-d G:i:s')."\n".'symfony '.$task."\n".$pakeweb->content);  // log task results
            } else {
                throw new Exception($pakeweb->error);
            }       

        }
                
        // Additional task to do after synchronization
        if ($sync) {            
                    
            // htaccess rebuilding and update
            $this->_htaccess(); 
            
            // deprecated option
            // if applications didn't synchronize properly, repeat step
            //            $apps_sync=$this->_appsToSync();
            //            if (sizeof($apps_sync)>0) return $step;
        } 
        
        // next step after synchronization is executed with new verion of this file (when stUpdate is upgraded)
        if ($task=='setup-update')
        {
            // fix synchronization known previous errors
            $this->fixSync();
        }
        
        return $step+1;
    }       
        
    /** 
     * Get message.
     *
     * @return   string
     */
    public function getMessage()
    {
        return $this->msg;
    }
    
    /** 
     * Last progress bar step.
     *
     * @param    int $opt 0 - means don't execute optimization after installation
     * @return   string
     */
    public function close($opt=1)
    {                    
        $i18n = sfContext::getInstance()->getI18N();
        
        if ($this->_installVerification())
        {                      
            if (class_exists('stLock'))
            {                         
                stLock::unlock('frontend');
                stLock::unlock('backend');  
            }  
            // unlock shop
        }     
        
        sfLoader::loadPluginConfig();
        
        $this->_postExecute($this->_loadAppsToSync('all'));  

        stLockUpdate::unlock();

        $this->msg = $i18n->__('Instalacja aktualizacji zakończona.');

        // cancel optimization (in stSetup for example)
        if ($opt==1)
        {  
            $this->addOptimization(); 
        }
        
        stEventDispatcher::getInstance()->notify(new sfEvent($this, 'stInstallerTaks.onClose', array()));

        $this->_cleanAppsToSync(); 
        $this->_cleanRegSync();   
    }
    
    /** 
     * Optimization and caching pages.
     * Delete download PEAR files. Optimize & cache actions.
     */
    private function addOptimization()
    {
        $this->msg  = progress_bar('stCleanInstallerDownload', 'stCleanInstallerDownload', 'step', stCleanInstallerDownload::getSteps());
        $this->msg .= progress_bar('stCleanInstallerSrc',      'stCleanInstallerSrc',      'step', stCleanInstallerSrc::getSteps());
        $this->msg .= progress_bar('stCleanInstallerCache',    'stCleanInstallerCache',    'step', stCleanInstallerCache::getSteps());
    }
    
    /** 
     * Set progress bar title.
     *
     * @return   string
     */
    public function getTitle()
    {
        $i18n = sfContext::getInstance()->getI18N();
        return $i18n->__('Instalacja aktualizacji (Uwaga! Nie zamykaj okna przeglądarki, aż instalacja się nie skończy):');
    }
                                
    /** 
     * Verify instllation.
     * @todo
     *
     * @return   bool
     */
    protected function _installVerification()
    {
        return true;
    }
    
    /** 
     * Build database diff.
     * Compare new version of database with current installation.
     *
     * @return   string
     */
    protected function _installer_schema_diff()
    {    
        $apps_sync=$this->_appsToSync();   
        $params=implode(' ',$apps_sync);  
        if (! empty($apps_sync)) 
        {
             return 'installer-schema-diff '.$params;
        } else return '';
    }
    
    /** 
     * Synchronization.
     *
     * @see stinstallerweb/syncList
     */
    protected function _sync()
    {                                      
        return 'installer-sync';
    }                                             
        
        
    /** 
     * Task: symfony installer-build-all
     * If package-schema-diff doesn't exist run propel-build-model.
     *
     * @return   string
     */
    protected function _installer_build_all() 
    {   
        $schema_diff_file=sfConfig::get('sf_config_dir').DIRECTORY_SEPARATOR.'package-schema-diff.yml';                                                                        
        if (file_exists($schema_diff_file)) return 'installer-build-all';
        else return 'propel-build-model';
    } 
          
    
    /** 
     * Get host.
     *
     * @return   string
     */
    protected function _getHost()
    {
        $context    = sfContext::getInstance();        
        $request    = $context->getRequest();  
        return $request->getHost();         
    }      
    
    /** 
     * Get application list for synchronization.
     *
     * @param                 string      $mode               (added|changed|deleted|all|apps)
     * @return   array
     */
    protected function _appsToSync($mode='all')
    {                              
        $regsync = new stRegisterSync();
        $apps=$regsync->getAppsToSync();   
        if ($mode=='apps')
        {    
            return $apps;
        } else return $apps[$mode];
    }
       
        
    /** 
     * Save log.
     *
     * @param        string      $message
     */
    protected function log($message)
    {                       
        if (sfConfig::get('sf_logging_enabled'))
        {
             $fd=fopen(ST_INSTALLER_LOG_PAGE,"a+");
             fwrite($fd,$message);
             fclose($fd);
        }                
    }
    
    /** 
     * Remember application list for synchronization.
     * Needed for steps after synchronization.
     *
     * @see ST_APPSTOSYNC_FILE
     * @return   bool
     */
    private function _saveAppsToSync()
    {   
        $apps=$this->_appsToSync('apps');               
        $data=serialize($apps);          
        if (file_put_contents(ST_APPSTOSYNC_FILE,$data)) return true;
        else 
        {
            throw new Exception('Unable to save data to file '.ST_APPSTOSYNC_FILE);
        }                  
        return false;
    }                                           
    
    /** 
     * Remember application list before upgrade.
     * Needed for steps afte upgrade.
     *
     * @return   bool
     */
    private function _saveRegSync()
    {             
        $reg = new stRegisterSync();
        $apps = $reg->getSynchronizedApps();    
        $data=serialize($apps);  
        if (file_put_contents(ST_REGSYNC_PRE_INSTALL_FILE,$data)) return true;
        else 
        {
            throw new Exception('Unable to save data to file '.ST_REGSYNC_PRE_INSTALL_FILE);
        }                  
        return false;
    }       
    
    private function _cleanRegSync()
    {                                
         if (file_exists(ST_REGSYNC_PRE_INSTALL_FILE))  unlink(ST_REGSYNC_PRE_INSTALL_FILE);
    }
    
    /** 
     * Odczytuje zapamiętane aplikacje przed synchronizacją
     * Get applications before upgrade.
     *
     * @see $this->_saveRegSync()
     * @return   array
     */
    private function _loadRegSync()
    {    
        if (file_exists(ST_REGSYNC_PRE_INSTALL_FILE))
        {
            $data=file_get_contents(ST_REGSYNC_PRE_INSTALL_FILE);
            $apps=unserialize($data);       
        } else $apps=array();
        
        return $apps;  
    }
                                              
    /** 
     * Load fixtures for provided applications.
     *
     * @param   array       $apps               application list
     */
    private function _propelLoadData($apps)
    {
        $pakeweb = new stPakeWeb();
        foreach ($apps as $app)
        {                                              
            $fixtures=$this->_getFixturePath($app);   
            foreach ($fixtures as $fixture)
            {                                               
                if (file_exists($fixture))
                {
                    $task="propel-load-data ../update $fixture append";
                    $md5_task_lock_file_name=md5($task).'_'.basename($fixture).'.lck';   // this file lock duplicate data loading
                    if (! stLockUpdate::isLockedKey($md5_task_lock_file_name)) 
                    {
                        if ($pakeweb->run($task))
                        {   
                            stLockUpdate::lockKey($md5_task_lock_file_name);            // lock data loading for the same app 
                            $pakeweb->content=$this->_fixcontent($pakeweb->content);                       
                            if (! empty($pakeweb->content)) $this->log("\n".date('Y-m-d G:i:s')."\n".'symfony '.$task."\n".$pakeweb->content);  
                        } else {
                            $this->log($pakeweb->error);
                        }
                    } else 
                    {
                         // data for this app was loaded
                    }
                }
            }            
        }
    }          
    
     /** 
      * Delete unimportant warning from logs.
      * Task propel-load-data return some warning which are ok, and should be savd in logs.
      * (warning about duplicated constat definitions)
      *
      * @param        string      $content
      * @return   string
      */
     private function _fixcontent($content)
     {
         $lines=split("\n",$content);
         $o='';
         foreach ($lines as $line)
         {
             if (ereg('already defined',$line)) continue;
             if (ereg('\<br \/\>',$line)) continue;
             $o.=$line."\n";
         }                                                       
         
         return $o;
     }
    
     /** 
      * Get fixture path in install/src/$app
      *
      * @param        string      $app
      * @param   array       array(/path/to/soteshop/data/fixtrures/stAppName.yml lub /path/to/soteshop/plugins/stAppNamePlugin/data/fixtures/stAppNamePlugin.yml)
      */
     private function _getFixturePath($app)
     {           
         $dir=sfConfig::get('sf_root_dir').DIRECTORY_SEPARATOR.'install'.DIRECTORY_SEPARATOR.'src'.DIRECTORY_SEPARATOR.$app;    
         $fixture_dirs=sfFinder::type('dir')->name('fixtures')->in($dir);

         $data=array();
         foreach ($fixture_dirs as $fd)              
         {                                              
             $files=sfFinder::type('file')->name($app.'.yml')->in($fd);                      
             foreach ($files as $file)
             {                                                                  
                 $d=split('install'.DIRECTORY_SEPARATOR.'src'.DIRECTORY_SEPARATOR.$app.DIRECTORY_SEPARATOR,$file,2);
                 if (! empty($d[1])) $data[]=sfConfig::get('sf_root_dir').DIRECTORY_SEPARATOR.$d[1];
             }
         }
         
         return $data;
     }
                                                            
    /** 
     * Get applications before synchronization.
     *
     * @see $this->_saveAppsToSync
     * @see ST_APPSTOSYNC_FILE
     * @param   string      $mode               type (added|changed|deleted)
     * @return  array       lista aplikacji
     */
    private function _loadAppsToSync($mode)
    {   
        $data=file_get_contents(ST_APPSTOSYNC_FILE);
        $apps=unserialize($data);       
        return $apps[$mode];  
    }    
            
    
    private function _cleanAppsToSync()
    {
        unlink(ST_APPSTOSYNC_FILE);
    } 
    
    /** 
     * Execute post install scripts.
     *
     * @param                 array       $apps               array('stAppName1','stAppName2',...)
     */
    private function _postExecute($apps)
    {   
        $peari = stPearInfo::getInstance();                    
        $regsync=$this->_loadRegSync();                                        
        
        foreach ($apps as $app)
        {                                
            $script = sfConfig::get('sf_data_dir').DIRECTORY_SEPARATOR.'update'.DIRECTORY_SEPARATOR.$app.'.php';       
            if (file_exists($script)) 
            {                          
                $version_new=$peari->getPackageVersion($app);;
                if (! empty($regsync[$app])) $version_old=$regsync[$app];                
                else $version_old=NULL;
                
                // lock another executing for the same app and version
                $md5_script_lock_file=md5($script).'_'.$app.'-'.$version_new.'.lck';
                if (! stLockUpdate::isLockedKey($md5_script_lock_file))
                {                                     
                    include_once($script);
                    stLockUpdate::lockKey($md5_script_lock_file); // lock execution for app and version
                }
            }
        }
    }
        
    /** 
     * Lock stSetup
     */
    private function _secureStSetup()
    {
        $file=sfConfig::get('sf_app_dir').DIRECTORY_SEPARATOR.'modules'.DIRECTORY_SEPARATOR.'stSetup'.DIRECTORY_SEPARATOR.'config'.DIRECTORY_SEPARATOR.'security.yml';	    
        // all:
        //  is_secure: on       	                 
        $data=array();
        $data['all']['is_secure']='on';
        file_put_contents($file,sfYaml::dump($data));
    }                      
    

    /** 
     * Rebuild & Update htaccess file.
     *
     * @param   bool        $force              true - always overwrite, false - only when stBase is upgraded
     */
    protected function _htaccess($force=false)
    {
        // Check if stBase was upgraded
        $apps=$this->_loadAppsToSync('changed');        
        if ((in_array('stBase',$apps)) || ($force))     
        {
            // previous version
            $htaccess_file_current=sfConfig::get('sf_web_dir').DIRECTORY_SEPARATOR.'.htaccess';
            // backup previous version
            copy($htaccess_file_current,sfConfig::get('sf_root_dir').DIRECTORY_SEPARATOR.'install'.DIRECTORY_SEPARATOR.'db'.DIRECTORY_SEPARATOR.'.htaccess.bk');
            $htaccess1 = new stHtaccess($htaccess_file_current);  
            // new version
            $htaccess_file_new=sfConfig::get('sf_root_dir').DIRECTORY_SEPARATOR.'install'.DIRECTORY_SEPARATOR.'src'.DIRECTORY_SEPARATOR.
                               'stBase'.DIRECTORY_SEPARATOR.'stBase'.DIRECTORY_SEPARATOR.'web'.DIRECTORY_SEPARATOR.'.htaccess';
            $htaccess2 = new stHtaccess($htaccess_file_new);         
            // merged files
            $htaccess3 = $htaccess_file_current; 

            if (file_exists($htaccess_file_new))
            {
                $merge = new stHtaccessMerge($htaccess1,$htaccess2);
                $merge->merge();
                $merge->save($htaccess3);
            }
        }
    }
    
    /** 
     * Fix upgrade previou known error.
     * 1. Error stNewsPlugin - delete old files stNewsPlugin/config/*
     */
    protected function fixSync()
    {
        // fix cache structore
        $fixcache = new stFixCache();
        $fixcache->fixAll();
        
        $src=sfConfig::get('sf_root_dir').DIRECTORY_SEPARATOR.'install'.DIRECTORY_SEPARATOR.DIRECTORY_SEPARATOR.'src';
        $src_stNewsPlugin = $src.DIRECTORY_SEPARATOR.'stNewsPlugin'.DIRECTORY_SEPARATOR.'plugins'.DIRECTORY_SEPARATOR.'stNewsPlugin';
        $st_stNewsPlugin  = sfConfig::get('sf_plugins_dir').DIRECTORY_SEPARATOR.'stNewsPlugin';
        // pliki do usuniecia
        $delete=array(
            $src_stNewsPlugin.DIRECTORY_SEPARATOR.'config'.DIRECTORY_SEPARATOR.'stNewsPlugin_stNewsPlugin-schema.custom.yml',
            $src_stNewsPlugin.DIRECTORY_SEPARATOR.'config'.DIRECTORY_SEPARATOR.'stNewsPlugin-schema.yml',
            $src_stNewsPlugin.DIRECTORY_SEPARATOR.'config'.DIRECTORY_SEPARATOR.'stNewsPlugin-schema.dbd.xml',
            $st_stNewsPlugin. DIRECTORY_SEPARATOR.'config'.DIRECTORY_SEPARATOR.'stNewsPlugin_stNewsPlugin-schema.custom.yml',
            $st_stNewsPlugin. DIRECTORY_SEPARATOR.'config'.DIRECTORY_SEPARATOR.'stNewsPlugin-schema.yml',
            $st_stNewsPlugin. DIRECTORY_SEPARATOR.'config'.DIRECTORY_SEPARATOR.'stNewsPlugin-schema.dbd.xml',
            );
        foreach ($delete as $file)
        {
            if (file_exists($file)) unlink($file);
        }
    }


    /**
     * Clear FastCache if needed
     */
    protected function clearFastCache()
    {
		foreach (glob(sfConfig::get('sf_root_dir').DIRECTORY_SEPARATOR.'fastcache'.DIRECTORY_SEPARATOR.'cache'.DIRECTORY_SEPARATOR.'cache*') as $cache_file)
		{
			unlink($cache_file);
		}
    	if (file_exists(sfConfig::get('sf_root_dir').DIRECTORY_SEPARATOR.'fastcache'.DIRECTORY_SEPARATOR.'db'.DIRECTORY_SEPARATOR.'fast_cache_generate_default')) {
			unlink(sfConfig::get('sf_root_dir').DIRECTORY_SEPARATOR.'fastcache'.DIRECTORY_SEPARATOR.'db'.DIRECTORY_SEPARATOR.'fast_cache_generate_default');    	
    	}
    }

   public function getFatalMessage()
    {
    	sfLoader::loadHelpers('I18N');
    	sfLoader::loadHelpers('Url');
    	$refresh_link = url_for('stInstallerWeb/rescueReboot',true);
    	return __("Wystąpił błąd podczas aktualizacji oprogramowania SOTESHOP 5.0, ponowna próba aktualizacji nastąpi za 30 sekund.");
    }    
}