<?php
/**
 * SOTESHOP  5.0.1
 * 
 * Ten plik należy do aplikacji SOTESHOP opartej na licencji (Professional License SOTE). 
 * Nie zmieniaj tego pliku, jeśli chcesz korzystać z automatycznych aktualizacji oprogramowania. 
 * Jeśli chcesz wprowadzać swoje modyfikacje do programu, zapoznaj się z dokumentacją, jak zmieniać 
 * oprogramowanie bez zmiany kodu bazowego http://www.sote.pl/modifications
 * 
 * @package     SOTESHOP
 * @copyright   SOTE (www.sote.pl)
 * @license     http://www.sote.pl/license/sote (Professional License SOTE)
 * @version     $Id: stLicenseInstaller.php 41 2009-08-24 14:51:53Z marek $
 * @author      Marek Jakubowicz <marek.jakubowicz@sote.pl>      
 * @since       stInstallerPlugin 1.0.3
 */

/**
 * Klasa stLicense
 * Sprawdzanie numeru i daty ważności licencji
 *
 * @package     soteshop_installer
 * @subpackage  libs    
 */
class stLicenseInstaller
{
    /**
     * Klucz licencji
     *
     * @var string
     */
    private $licenseKey = '071174180986080181986863209823MK';

    /**
     * Sprawdzanie numeru licencji
     *
     * @param string $license
     * @return bool
     */
    public function checkLicense($license)
    {
        $license = $this->clearLicense($license);
        $licenseLenght = strlen($license);
        if ($licenseLenght != 24) return false;

        $licenseNoSing = substr($license, 0, $licenseLenght-8);
        $licenseSing = substr($license, $licenseLenght-8, 8);

        $code = md5($this->licenseKey.$licenseNoSing);
        $codeSing = substr($code, strlen($code)-8, 8);

        if ($codeSing == $licenseSing)
        {
            if($this->checkLicenseDate($license)) return true;
        }
        return false;
    }

    /**
     * Pobieranie daty wygenerowania licencji w timestamp
     *
     * @param string $license
     * @return int
     */
    private function getLicenseCreateDate($license)
    {
        $license = $this->clearLicense($license);

        $year = substr($license, 0, 4);
        $month = substr($license, 4, 2);
        $day = substr($license, 6, 2);

        return mktime(0, 0, 0, $month, $day, $year);
    }

    /**
     * Pobieranie ważności licencji w dniach
     *
     * @param string $license
     * @return integer
     */
    private function getLicenseDayLimit($license)
    {
        $license = $this->clearLicense($license);
        return substr($license, 8, 2);
    }

    /**
     * Sprawdzanie ważności licencji
     *
     * @param string $license
     * @return bool
     */
    private function checkLicenseDate($license)
    {
        $license = $this->clearLicense($license);

        $licenseCreateDate = $this->getLicenseCreateDate($license);
        $licenseDayLimit = $this->getLicenseDayLimit($license);

        if ($licenseDayLimit == 0) return true;

        $currentDate = mktime(0, 0, 0, date('n'), date('j'), date('Y'));
        $licenseLimitDate = $licenseCreateDate + ($licenseDayLimit*24*60*60);

        if($currentDate < $licenseLimitDate) return true;
        return false;
    }

    /**
     * Usuwanie spacji i myślinków z numeru licencji
     *
     * @param string $license
     * @return string
     */
    private function clearLicense($license)
    {
        $license = ereg_replace(" ", "", $license);
        $license = ereg_replace("-", "", $license);
        return $license;
    }
}