<?php
/**
 * SOTESHOP  5.0.1
 * 
 * Ten plik należy do aplikacji SOTESHOP opartej na licencji (Professional License SOTE). 
 * Nie zmieniaj tego pliku, jeśli chcesz korzystać z automatycznych aktualizacji oprogramowania. 
 * Jeśli chcesz wprowadzać swoje modyfikacje do programu, zapoznaj się z dokumentacją, jak zmieniać 
 * oprogramowanie bez zmiany kodu bazowego http://www.sote.pl/modifications
 * 
 * @package     SOTESHOP
 * @copyright   SOTE (www.sote.pl)
 * @license     http://www.sote.pl/license/sote (Professional License SOTE)
 * @version     $Id: stSoteshopInstallerCli.php 8339 2010-09-17 08:14:50Z marek $
 * @author      Marek Jakubowicz <marek.jakubowicz@sote.pl>      
 * @since       stInstallerPlugin 1.0.3
 */                         
      
/**                 
 * Instalator SOTESHOP
 * 
 * @package SOTESHOP
 */
class stSoteshopInstallerCli
{   
    /**
     * Domyślny kanał PEAR
     */                   
    var $pear='pear.sote.pl';
    
    /**
     * @var string database=username:password@localhost/dbname
     */                                                       
    var $database;
    
    /**
     * @var string Unix|Windows
     */     
    var $system;  
    
    /**
     * @var array dane do stAurthUsers
     */           
    var $auth=array();          
    
    /**
     * @var string głowna instalowana aplikacja
     */                               
    var $application;  
    
    /**
     * @var array lista instalowanych aplikacji
     */                                        
    var $applications=array();     
    
    /**
     * Wartość true oznacza pobranie z PEAR symfony do lokalnego katalogu
     */                                                                  
    var $symfonyFreeze=false;                    
    
    /**
     * @var string ścieżka do pliku z baza instalowanych aplikacji
     */           
    var $dbfile;   
    
    /**
     * Określa lokalizacje pobranych pakietów PEAR
     */                                           
    var $download;             
    
    /**
     * Lokalizacja stInstallerPlugin (wg parametrow pear install)
     */                                                      
    var $stinstallerplugin;
    
    /**
     * Mirror PEAR
     */           
    // var $pearMirror;     
    
    /**
     * Plik z logami instalacji.
     */
    var $logFile='log/cliinstaller.log';
     
    public function __construct()
    {       
        $this->checkDirectory();                                          
        $this->systemSettings();
        $this->checkParams();    
        $this->checkDB();
        $this->checkPHP();       
        $this->checkPEAR();
        $this->checkSymfony();        
        $this->initSymfony();
        $this->initFs();          
        
        $this->initPEAR();                      
    }                                    

    /**
     * Ustawienia dla systemu operacyjnego
     *
     */
    protected function systemSettings() {
        $system_uid=posix_getuid();
        $script_uid=getmyuid(); $script_user=get_current_user();
        if ($system_uid==0) die ('This script can\'t be run as user root.'."\n");
        if ($system_uid!=$script_uid) die ('The script must be run as '.$script_user." user.\n");   
    }
    
    /**
     * Weryfikuje użytkownika systemowego uruchamiającego instalację oraz system operacyjny.
     */
    public static function checkSystem()
    {                                                                  
        if (! eregi("^WIN",PHP_OS)) {                 
            return 'Unix';     
        } else 
        {                        
            return 'Windows';   
        }
        
    }                      
            
    /**
     * Weryfikuje i odczytuje parametry wejściowe.
     * Parametry channel/stAppName database=username:password@localhost/dbname auth=username:password pkgdb=/path/to/packagesdb.yml
     * 
     * @see $this->database      
     */
    protected function checkParams()
    {          
        $argv=$_SERVER['argv'];    
        $i=0;                                       
        
        if (empty($argv[1])) die($this->help());
                                                   
        foreach ($argv as $arg)
        {   
            if ($i>0)
            {   
                if (ereg('=',$arg)) {
                    $dat=split('=',$arg,2);

                    $option=$dat[0];
                    $value=$dat[1];             

                    switch ($option)
                    {   
                        case 'database': 
                        $this->_database($value);
                        $this->database='database='.$value;
                        break;
                        case 'auth': $this->_auth($value);
                        break;   
                        case 'dbfile':
                        $this->_dbfile($value); $this->dbfile=$value;
                        break; 
                        case 'application':
                        $this->_application($value);
                        break;
                        case 'channel':
                        $this->pear=$value;
                        break;  
                        case 'download':
                        $this->download=$value;
                        break;      
                        case 'symfony';
                        if ($value=='freeze') $this->symfonyFreeze=true;
                        break;  
                        case 'license':
                        $this->_license($value);
                        break;                        
                        case 'pear':
                        if ($value=='local') $this->_installpear();
                        break;        
                        case 'installer':
                        $this->stinstallerplugin=$value;
                        break;
                        
                        // case 'mirror':
                        //                      $this->pearMirror=$value;
                        //                      break;      
                    }
                } else die ($this->help());        
            }
            $i++;
        }         
        
        // sprawdz czy podana została aplikacja lib lista aplikacji.
        if ((empty($this->application)) && (empty($this->applications))) die($this->help());
        
        if (empty($this->database)) die ($this->help());                        
        
        // sprawdz czy podano licencje; licencja nie jest wymagana dla channel=pear.dev.quad.sote.pl           
        if ($this->pear!='pear.dev.quad.sote.pl')
        {
            if ((empty($this->license)) && ($this->isSoteshop())) die ("\n".'Wrong license number.'.$this->help());
        }
     }        
    
    /**
     * Zwraca hash losowy
     * @return string
     */
    private function getHash()
    {
        return md5(md5(time()));
    }
    
    /**
     * Parametr: license    
     *
     * @param string $license
     */  
    protected function _license($value)
    {                                           
        $license = new stLicenseInstaller($value);
        if ($license->checkLicense($value)) {
            $this->license=$value;           
            $shop_hash=$this->getHash();
            $o='';
            $o.="---\n";
            $o.="all:\n";
            $o.="  .auto_generated:\n";
            $o.="    config:\n";
            $o.="      last_modified: ".date('Y/m/d H:m:s')."\n";
            $o.="      fields:\n";
            $o.="        license: $value"."\n";
            $o.="        shop_hash: $shop_hash"."\n";
            
            if (! is_dir(SF_ROOT_DIR.DIRECTORY_SEPARATOR.'data')) mkdir (SF_ROOT_DIR.DIRECTORY_SEPARATOR.'data');
            if (! is_dir(SF_ROOT_DIR.DIRECTORY_SEPARATOR.'data'.DIRECTORY_SEPARATOR.'config')) mkdir (SF_ROOT_DIR.DIRECTORY_SEPARATOR.'data'.DIRECTORY_SEPARATOR.'config');
            $file=SF_ROOT_DIR.DIRECTORY_SEPARATOR.'data'.DIRECTORY_SEPARATOR.'config'.DIRECTORY_SEPARATOR.'__stRegister.yml';
            
            file_put_contents($file,$o);                                                                                  
        }
    }  
    
    /**
     * Parametr: application
     *
     * @param string $application
     * @see $this->pear
     * @see $this->application
     */                       
     protected function _application($value)
     {
         if (file_exists($value)) {
             $this->application=$value;
             return;
         }
                    
         if (ereg('/',$value))
         {
             $dat=split('/',$value);
             $this->pear=$dat[0];
             $this->application=$dat[1];
         } else $this->application=$value;
     }
    
    /**
     * Parametry: 'database'
     *
     * @param string $value      
     * @see $this->db
     */
    protected function _database($value)
    {
        if ((! eregi("^[0-9a-z_-]+\:[0-9a-z_-]+\@[a-z0-9_\.-]+\/[a-z0-9_-]+",$value)) and 
            (! eregi("^[0-9a-z_-]+\@[a-z0-9_\.-]+\/[a-z0-9_-]+",$value))
           ) {
            die($this->help());
        }                      
        
        preg_match("/^([0-9a-z_-]+)\:([0-9a-z_-]+)\@([a-z0-9_\.-]+)\/([a-z0-9_-]+)/i",$value,$matches);      
        if (! empty($matches)) {
            $this->db['username']=$matches[1];
            $this->db['password']=$matches[2];
            $this->db['host']=$matches[3];
            $this->db['dbname']=$matches[4];            
        } else {
            preg_match("/^([0-9a-z_-]+)\@([a-z0-9_\.-]+)\/([a-z0-9_-]+)/i",$value,$matches); 
            $this->db['username']=$matches[1];
            $this->db['host']=$matches[2];
            $this->db['dbname']=$matches[3]; 
            $this->db['password']='';    
        }
    }   
        
    /**
     * Parametry: auth
     *
     * @param string $value           
     * @see $this->auth
     */
    protected function _auth($value)
    {
        if (ereg(':',$value))
        {
            $dat=split(':',$value);  
            $this->auth['username']=$dat[0]; 
            $this->auth['password']=$dat[1];            
        } else die($this->help());
    }               
    
    /**
     * Odczytuje listę aplikacji do zainstalowania
     *
     * @param string $value
     */                    
    protected function _dbfile($value)
    {
        if (file_exists($value)) {
            $data=$this->_getYaml($value);        
            if (is_array($data))
            {
               $this->applications=$data;  
            } else die("Wrong file format in the $value.\n".$this->help());
        } else die("The file ".$value." not found.\n".$this->help());
    }        
    
    /**
     * Odczytuje dane z pliku YAML. 
     * Rozpoznaje tablicę jednowymiarową.
     *
     * @param string $file
     */                   
    protected function _getYaml($file)
    {       
        $res=array();
        
        if (empty($file)) return $res;
        if (! file_exists($file)) return $res;
        
        $data=file_get_contents($file);
        $lines=split("\n",$data);
        foreach ($lines as $line) {
            if (ereg(':',$line)) {
                $dat=split(':',$line);
                $application=trim($dat[0]);
                $version=trim($dat[1]);   
                $res[$application]=$version;    
            }                          
            
        }                               
        return $res;
    }
    
    /**
     * Pomoc dla komendy instalatora.
     * 
     * @return string 
     */                
     protected function help()
     {   
         $argv=$_SERVER['argv'];  
         $help="\n";  
         $help.="Try php ".$argv[0]." [option=value]\n";
         $help.="\n";
         $help.="Example:\n";
         $help.="  php ".$argv[0]." license=XYZ application=soteshop database=username:password@localhost/dbname auth=admin@example.com:password\n";        
         $help.="\n";
         return $help;
     }           

     /**
      * Zwarca true jeśli instalowany jest pakiet soteshop
      * @return bool
      */
     protected function isSoteshop()
     {
         if ($this->application=='soteshop') return true;    
         if (! empty($this->applications['soteshop'])) return true;
         return false;
     }                          
     
                                                   
      
         
     /**
      * Zwaraca listę aplikacji do zainstalowania
      */
     public function getApplications()
     {                             
         if (empty($this->applications))   return array($this->application=>'latest');
         else                              return $this->applications;
     }
    
    /**
     * Weryfikuje PHP
     */              
    protected function checkPHP()
    {
        $this->checkVersion(phpversion(),ST_PHP_MIN,ST_PHP_MAX,'PHP');          
        $this->log("PHP: ... OK");
    }
        
    /**
     * Weryfikuje wersje PEAR
     */
    protected function checkPEAR()
    {
        @include('PEAR.php');
        if ((! defined('PEAR_OS')) || (! class_exists('PEAR'))) die ('The PEAR Repository not found in include_path.'."\n");     
        $this->log("PEAR: ... OK");
    }       
    
    /**
     * Weryfikuje połączenie z bazą danych
     */      
    protected function checkDB()
    {   
        $con=@mysql_connect( $this->db['host'], $this->db['username'], $this->db['password']) or die('Could not connect to mysql server.'."\n" );
        @mysql_select_db($this->db['dbname'], $con) or die('Could not select database.'."\n");
        $this->log("Database connection: ... OK");

    }           
        
    /**
     * Weryfikuje wersję Symfony
     */
    protected function checkSymfony()
    {   
        $include_path=ini_get('include_path');
        $paths=split(':',$include_path);
        foreach ($paths as $path)
        {                   
            $file=$path.DIRECTORY_SEPARATOR.'symfony'.DIRECTORY_SEPARATOR.'VERSION';
            if (file_exists($file))
            {
                $symfony_version_file=$file;
            }
        }                                   
        
        if (empty($symfony_version_file))  die('The symfony not found in PEAR repository.'."\n");
        
        $fd=fopen($symfony_version_file,'r');$raw_version=fread($fd,filesize($symfony_version_file));fclose($fd);
        $version=trim($raw_version);        
        $this->checkVersion($version,ST_SYMFONY_MIN,ST_SYMFONY_MAX,'symfony');
        
        $command=$this->symfony('global').' -V';

        $symfony=exec($command);
        if (! eregi('version',$symfony)) die ("Check symfony installation.\n");
        $this->log("Symfony: ... OK");
    }  
        
    /**
     * Weryfikuje wersję
     *
     * @param string $version     np. 1.0.29
     * @param string $version_min np. 1.0.10
     * @param string $version_max np. 1.0.30                                 
     * @param string $name        nazwa elementu wersji
     */
    protected function checkVersion($version,$version_min,$version_max,$name='')
    {
        $v=intval(str_replace('.','',$version));    
        $v_min=intval(str_replace('.','',$version_min));    
        $v_max=intval(str_replace('.','',$version_max));            
        if (! (($v>=$v_min) && ($v<=$v_max))) die ('The '.$name.' version is '.$version.'. Required min: '.$version_min.'  max: '.$version_max."\n");   
        return true;
    }                 
        
    /**
     * Weryfikuje katalog, w którym instalowane jest oprogramowanie.
     */
    protected function checkDirectory()
    {                                    
        if (file_exists('symfony')) die ('The SOTESHOP project already exists in this directory.'."\n");
    }     
    
    /**
     * Zwraca komendę symfony dla systemu operacyjnego.
     * @param string $global - symfony wywolywane globalnie
     */                                                    
    protected function symfony($global=NULL)
    {                                      
        if ($global != "global") $symfony=$this->php().' symfony.php ';
        else {
            if (file_exists('/usr/bin/symfony')) $symfony='/usr/bin/symfony';
            elseif (file_exists('/usr/local/bin/symfony')) $symfony='/usr/local/bin/symfony';
            else $symfony="symfony";   
        }
                
        
        return $symfony;
    }               
        
    /**
     * Zwraca komendę pear  dla systemu operacyjnego.
     */
    protected function pear()
    {
       if (file_exists('/usr/bin/pear')) $pear='/usr/bin/pear';
       elseif (file_exists('/usr/local/bin/pear')) $pear='/usr/local/bin/pear';
       else $pear='pear';
                     
       return $pear; 
    }            
    
    /**
     * Zwaraca ścieżkę do php.
     */                       
     protected function php()
     {                           
         $inifile=SF_ROOT_DIR.DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'phpini.php';     

         if (file_exists($inifile))
         {
             include($inifile);
             if (isset($ini['php'])) $php=$ini['php'];
         } else 
         {
             if (file_exists('/usr/bin/php')) $php='/usr/bin/php';
             elseif (file_exists('/usr/local/bin/php')) $php='/usr/local/bin/php';
             else $php='php';
         }                  
         
         return $php;
    }            
         
     /**
      * Wykonuje komendę systemową.
      * @param string $exec komenda systemowa
      */
     protected function exec($exec)
     {    
         $this->log("exec:   $exec");                                   
         $ret=0;
                  
         ob_start();          
         system($exec, $ret);           
         $content = ob_get_clean();             
         $this->log($content);
         
         if ($ret>0) {         
             if (PEAR::isError($ret)) {
                 die ($result->getMessage() . "\n\n"); 
             } else {
                 die ("\n\n".'Installation error. Check '.$this->logFile."\n\n");
             }
         }   
         
         return $content;
     }
    
    /**
     * Inicjuje projekt Symfony
     */                        
     protected function initSymfony()
     {
         $this->exec($this->symfony('global').' init-project soteshop');
         if ($this->symfonyFreeze) $this->exec($this->symfony('global').' freeze');
         if (file_exists(SF_ROOT_DIR.DIRECTORY_SEPARATOR.'symfony'))
         {
             $this->log("Symfony project SOTESHOP created");
         } else {
             die ('The symfony project wasn\'t created.'."\n");
         }
     }                   
     
     /**
      * Modyfikuje stukturę plików/katalogów.
      */                          
     protected function initFs()
     {                            
         unlink (SF_ROOT_DIR.DIRECTORY_SEPARATOR.'config'.DIRECTORY_SEPARATOR.'schema.yml');
         $dirs=array( 
             'cache/src'=>0777,
             'cache/smarty_c'=>0777,
             'install'=>0777,
             'install/src'=>0777,
             'install/cache'=>0777,
             'install/download'=>0777,
             'install/db'=>0777,
             );
                                   
         foreach ($dirs as $dir=>$mode)
         {
             $dir_path=SF_ROOT_DIR.DIRECTORY_SEPARATOR.str_replace('/',DIRECTORY_SEPARATOR,$dir);             
             if (! is_dir($dir_path)) 
             {
                 mkdir($dir_path,$mode);
             }
             if (! is_dir($dir_path)) die ('Can\'t create the directory '.$dir."\n");             
         }                     
         
         // dodaj skrypt symfony.php bedacy kopia ./symfony  z pominiem wywolania powloki
         $fd=fopen('symfony','r');
         $data=fread($fd,filesize('symfony'));
         fclose($fd);
         $src=split("\n",$data,100);$src2='';
         foreach ($src as $line)
         {
             if (! ereg('\#\!\/',$line)) $src2.=$line."\n";
         }                                                 
         $src2=substr($src2,0,strlen($src2)-1);
         file_put_contents('symfony.php',$src2);
         // end                                                         
     }      
     
     /**
      * Konfiguruje PEAR.
      */                  
     protected function initPEAR()
     {            
         $this->log("PEAR setup:");       
         $ROOTDIR=SF_ROOT_DIR;                          
         $peardir=$ROOTDIR.DIRECTORY_SEPARATOR.'install'.DIRECTORY_SEPARATOR.'src';              
         $pearfile='.pearrc';                             
         $pearrc=$peardir.DIRECTORY_SEPARATOR.$pearfile;                                
         
         $pearrc_plugins=$ROOTDIR.DIRECTORY_SEPARATOR.'plugins'.DIRECTORY_SEPARATOR.'.pearrc';
         
         $execs=array(                    
             $this->symfony().' plugin-list',               
             $this->pear()." -c $pearrc_plugins config-set  temp_dir    $ROOTDIR/cache",
                          
             'cd plugins; '.$this->pear().' -c .pearrc channel-discover '.$this->pear.';cd ..',
             'cd install'.DIRECTORY_SEPARATOR.'src; '.$this->pear().' config-create '.$peardir.' '.$pearfile,                                   
             $this->pear()." -c $pearrc config-set php_dir      $ROOTDIR/install/src",
             $this->pear()." -c $pearrc config-set bin_dir      $ROOTDIR/install/src",
             $this->pear()." -c $pearrc config-set doc_dir      $ROOTDIR/install/src",
             $this->pear()." -c $pearrc config-set data_dir     $ROOTDIR/install/src",
             $this->pear()." -c $pearrc config-set ext_dir      $ROOTDIR/install/src",
             $this->pear()." -c $pearrc config-set test_dir     $ROOTDIR/install/src",
             $this->pear()." -c $pearrc config-set cfg_dir      $ROOTDIR/install/src",
             $this->pear()." -c $pearrc config-set www_dir      $ROOTDIR/install/src",
             $this->pear()." -c $pearrc config-set temp_dir     $ROOTDIR/install/cache",
             $this->pear()." -c $pearrc config-set cache_dir    $ROOTDIR/install/cache",
             $this->pear()." -c $pearrc config-set download_dir $ROOTDIR/install/download",                                                                        
             $this->pear()." -c $pearrc config-set cache_ttl        3600",
             $this->pear()." -c $pearrc config-set php_bin      \"".$this->php()."\"",     
             $this->pear()." -c $pearrc config-set preferred_state  stable",
             $this->pear()." -c $pearrc config-set umask            022",                         
             );    
               
             
         foreach ($execs as $exec)
         {           
             $exec=str_replace('/install/',DIRECTORY_SEPARATOR.'install'.DIRECTORY_SEPARATOR,$exec);
             $this->exec($exec);
         }   
        
         // zainstaluj instalator SOTESHOP
         $this->installStInstallerPlugin();       
         $this->exec($this->symfony().' cc');
                            
         $this->exec($this->pear().' -c install'.DIRECTORY_SEPARATOR.'src'.DIRECTORY_SEPARATOR.'.pearrc channel-discover '.$this->pear);
         $this->exec($this->pear().' -c install'.DIRECTORY_SEPARATOR.'src'.DIRECTORY_SEPARATOR.'.pearrc config-set default_channel '.$this->pear);
                           
     }                        
         
     /**
      * Instaluje stInstallerPlugin
      */
     protected function installStInstallerPlugin()
     {                                                                                                
         $data=$this->_getYaml($this->dbfile);
         if (! empty($data['stInstallerPlugin']))
         {                                  
             if (empty($this->download))
             {
                 $exec=$this->symfony(). ' plugin-install http://'.$this->pear.'/get/stInstallerPlugin-'.$data['stInstallerPlugin'].'.tgz';              
             } else {
                 $exec=$this->symfony(). ' plugin-install '.$this->download.DIRECTORY_SEPARATOR.'stInstallerPlugin-'.$data['stInstallerPlugin'].'.tgz';                  
             }
         } else 
         {
             $exec=$this->symfony(). ' plugin-install '.$this->pear.'/stInstallerPlugin';              
         }     
                                              
         
         if (! empty($this->stinstallerplugin))
         {
            $exec=$this->symfony(). ' plugin-install '.$this->stinstallerplugin;   
         }
         $this->exec($exec); 
        
         if (! is_dir(SF_ROOT_DIR.DIRECTORY_SEPARATOR.'plugins'.DIRECTORY_SEPARATOR.'stInstallerPlugin')) 
         {
             die ('  The stInstallerPlugin wasn\'t installed.'."\n");
         } else 
         {
             $this->log("  Application stInstallerPlugin installed");
         }       
     }   
         
     /**
      * Instaluje aplikację SOTESHOP  
      *
      * @param string $app  aplikacja
      * @param string $ver  wersja
      */                            
     public function install($app,$ver=NULL)
     {                                   
         if ($app=='stInstallerPlugin') return; 
         ob_start();          
         if ((empty($ver)) or ($ver=='latest')) 
         {
             system($this->pear().' -c install'.DIRECTORY_SEPARATOR.'src'.DIRECTORY_SEPARATOR.'.pearrc install '.$app);              
         } else 
         {                                                              
             // instaluj konkretną wersję bez zależności pakietowych        
             if (empty($this->download))
             {   
                 // instaluje z serwera PEAR
                 system($this->pear().' -c install'.DIRECTORY_SEPARATOR.'src'.DIRECTORY_SEPARATOR.'.pearrc install -n http://'.$this->pear.'/get/'.$app.'-'.$ver.'.tgz');
             } else
             {
                 // instaluje z pobranych wcześniej pakietów
                 system($this->pear().' -c install'.DIRECTORY_SEPARATOR.'src'.DIRECTORY_SEPARATOR.'.pearrc install -n '.$this->download.DIRECTORY_SEPARATOR.$app.'-'.$ver.'.tgz');
             }
         }
         $content = ob_get_clean();             
         $this->log($content);
     }
         
     /**
      * Task installer-sync
      */
     public function sync() 
     {                                            
         $this->exec($this->symfony().' installer-sync');       
     }
     
     /**
      * Task installer-schema-diff
      */
     public function diff() 
     {                                            
         $this->exec($this->symfony().' installer-schema-diff');       
     }                   
     
     /**
      * Task setup database=user:pass@host/dbname
      */
     public function setup() 
     {                                            
         $this->exec($this->symfony().' setup '.$this->database);       
     }                                           
     
     /**
      * Task installer-build-all
      */
     public function buildall() 
     {          
         if ($this->checkSchema()) $this->exec($this->symfony().' installer-build-all');       
     }                      
     
     /**
      * Laduje domyślne dane (fixtures)
      */    
     public function load()
     {         
         if ($this->isSoteshop()=='soteshop')
         {
             $this->exec($this->symfony(). " propel-load-data backend");
         }
     }      
     
     /**
      * Dodaje autoryzacje so stAuthUsers
      */                                 
     public function setAuth()
     {                                           
         if ($this->isSoteshop()=='soteshop') {
             $this->exec($this->symfony(). " add-auth-user ".$this->auth['username'].' '.$this->auth['password']);  
         }
     }                
      
     /**
      * Zapisuje komunikaty systemowe.
      *
      * @param string $message              
      * @param string $title    nazwa komunikatu, tytuł
      */       
     protected function log($message,$title='')
     {                    
         $this->logFile=str_replace('/',DIRECTORY_SEPARATOR,$this->logFile);    
         $log_dir=dirname(SF_ROOT_DIR.DIRECTORY_SEPARATOR.$this->logFile);        
         if (! is_dir($log_dir)) {
             if (! mkdir($log_dir)) die ('Unable to make dir: '.$log_dir."\n");
         }
         if (! empty($message)) {             
             $fd=fopen($this->logFile,'a+');
             fwrite($fd,date('Y-m-d H:i:s').'  '.$message."\n");
             fclose($fd);
         }
     }       
         
     /**
      * Weryfikuje i naprawia uprawnienia systemu plikow.
      */
     public function fixperm()
     {
         if ($this->isSoteshop())
         {
             $this->exec($this->symfony(). "st-fix-perm");
         } else 
         {
             $this->exec($this->symfony(). " fix-perm");
         }
     }           
     
     /**
      * Nadpisuje .htaccess wygenerowany przez Symfony
      */                                              
     public function fixhtaccess()
     {        
         $dir=SF_ROOT_DIR.DIRECTORY_SEPARATOR.'web'.DIRECTORY_SEPARATOR;
         copy($dir.'htaccess',$dir.'.htaccess');
         unlink($dir.'htaccess');
     }
     
     /**
      * Instaluje lokalnie repozytorium PEAR
      */                                    
     protected function _installpear()
     {                
         $peardir=SF_ROOT_DIR;   
         $pearfile='.pearrc';    
         $pearrc=$peardir.DIRECTORY_SEPARATOR.$pearfile;
         
         $execs=array(                                
              'cd '.$peardir.'; pear config-create '.$peardir.' '.$pearfile,                 
              $this->pear()." -c $pearrc install PEAR-1.7.2",                         
              );    

          foreach ($execs as $exec)
          {           
              $exec=str_replace('/install/',DIRECTORY_SEPARATOR.'install'.DIRECTORY_SEPARATOR,$exec);
              $this->exec($exec);
          }
     }      
     
     /**
      * Sprawdza czy są jakieś schematy bazy danych.
      * Czy zainstalowane programy zawierają obsługę bazy danych.    
      * Weryfikacja polega na odczytaniu wygenerowanej różnicy bazy danych.
      *
      * @return bool 
      */
     protected function checkSchema()
     {        
         $file=SF_ROOT_DIR.DIRECTORY_SEPARATOR.'config' .DIRECTORY_SEPARATOR.'package-schema-diff.yml';
         if (! file_exists($file)) return false;
         else 
         {                  
             $data=file_get_contents($file);
             if (ereg('propel',$data)) return true;
             else return false;
         }
     }

    /**
     * Optymalizuje ilosc plikow
     */
    public function clean()
	{
        $this->exec($this->symfony(). "installer-clean");
    }
     
     /**
      * Wyświetla komunikat o powodzeniu instalacji
      */                                            
     public function success()
     {
         echo "\n\n";
         echo "|========================================|\n";
         echo " Congratulations!                         \n"; 
         if (! empty($this->application)) echo " The ".$this->application." is installed. \n";     
         else echo " Software is installed\n";
         echo "|========================================|\n\n";
     }
}      


         
