<?php           
/** 
 * SOTESHOP/stUpdate 
 * 
 * This file is the part of stUpdate application. License: (Open License SOTE) Open License SOTE. 
 * Do not modify this file, system will overwrite it during upgrade.
 * 
 * @package     stUpdate
 * @subpackage  actions
 * @copyright   SOTE (www.sote.pl)
 * @license     http://www.sote.pl/license/open (Open License SOTE) Open License SOTE
 * @version     $Id: actions.class.php 4624 2010-04-19 17:39:33Z marek $
 * @author      Marek Jakubowicz <marek.jakubowicz@sote.pl>
 */
                                     
define ("ST_MIN_EXECUTION_TIME",20);       // minimalny czas potrzebny do wykonania aktualziacji
define ("ST_MIN_EXECUTION_TIME_APP",0.1);  // minimalny dodatkowy czas aktualizacji 1 pakietu

/** 
 * stInstallerWeb actions.
 *
 * @author Marek Jakubowicz <marek.jakubowicz@sote.pl>
 *
 * @package     stUpdate
 * @subpackage  actions
 */
class stInstallerWebActions extends sfActions
{                                     
    
    /** 
     * Executes index action
     */
    public function executeIndex()
    {
       $server = new stNewServer();
       if ($server->newServer())
       {
           $this->redirect('installerweb/newServer'); 
       } else 
       {
           $this->redirect('installerweb/upgradeList');
       }
    } 
    
    /**
     * Execute newServer action
     */
    public function executeNewServer()
    {
        $pakeweb = new stPakeWeb();
        $pakeweb->run('cc');
        $server = new stNewServer();
        $server->update();
    }
    
    /** 
     *  List of installed applications.
     */
    public function executeList()
    {         
        $regsync = new stRegisterSync();
        $this->content=$regsync->getSynchronizedApps();         
    }  
    
    /** 
     * List of PEAR installer applications.
     */
    public function executeListpear()
    {
        ob_start();
        
        $installer_web = new stInstallerFrontendWeb();            
        $installer_web->command('list');
        
        $content = ob_get_clean(); 
        $this->content = stPearFrontendWeb::getPearResult($content);


    }                 
    
    
    /** 
     * Executes install action
     */
    public function executeInstall()
    {
        if (!stCheckRequirements::test()) {$this->redirect('stInstallerWeb/requirements');}
        stRegisterSync::fixmd5sum(); 
                      
        $channel=$this->getRequestParameter('channel');
        $package=$this->getRequestParameter('package');
        
        ob_start();
        
        $installer_web = new stInstallerFrontendWeb();            
        $installer_web->command('install',array("$channel/$package"));
                                        
        $content = ob_get_clean();
        $content=stPearFrontendWeb::getPearResult($content);

        // fix bledu PEAR : in PEAR result <div> is not closed, fix it
        if (ereg("\<div\>$",$content)) $content=preg_replace("/\<div\>$/","</div>",$content);     
        
        $this->content=$content;    
    }  
    
    public function executeRequirements() {
        $this->curl = stCheckRequirements::testCurl();
        $this->suPHP = stCheckRequirements::testSuPHP();
        $this->safeMode = stCheckRequirements::testSafeMode();
        $this->Json = stCheckRequirements::testJsonEncode();
    }
    
    /** 
     * Executes updatelist action
     * If list contain stUpdate, show only stUpdate.
     */
    public function executeUpgradeList()
    {
           
        if (!stCheckRequirements::test()) {$this->redirect('stInstallerWeb/requirements');}

        ob_start();
      
        $installer_web = new stInstallerFrontendWeb();                    
        $installer_web->command('list-upgrades');
        
        $content = ob_get_clean();
        $content=stPearFrontendWeb::getPearResult($content);

        if (is_array($content)) $this->upgrade=true;
        else 
        {
            $this->content=array();
            $this->upgrade=false;     
        }
        
        // $this->upgrade=true; // tmp          
        
        // jesli jednym z apkietow jest stUpdate, to nie pokazuj innych    
        $this->installerweb=false;    // czy w pakietach do aktualizacji jest stWebInstallerPlugin
        $installer_web=false;         
        if ($this->upgrade==true)
        {
            foreach ($content as $id=>$data)
            {
                if ($data['name']=='stUpdate') 
                {
                    $installer_web=true;
                    $installer_id=$id;
                }
            }                          

            if ($installer_web)
            {   
                $this->installerweb=true;
                $content=array(0=>$content[$installer_id]);
            }
            // end
        }
                      
        $this->content=$content;
        
        
        // lista aplikacji oczkujacych na synchronizacje
        $regsync = new stRegisterSync();
        $apps_sync=$regsync->getAppsToSync();      
        $this->sync_apps=$apps_sync['all'];   
        
        // jesli stInstallerWebPlugin  oczekuje na synchronizacje, to tez nie pokazuj innych pakietów      
        $installer_web_sync=false;
        foreach ($apps_sync['all'] as $id=>$name)
        {
            if ($name=='stUpdate') 
            {
                $installer_web_sync=true;
            }
        }
                               
        if ($installer_web_sync)
        {
            $this->content=array();
        } 
                                                  
    }
    
    /** 
     * Executes upgrade action
     *
     * @param   bool        $all                true - upgrade all available packages, false - upgrade packages dependency for 1 package
     */
    public function executeUpgrade($all=false)
    {   
        stRegisterSync::fixmd5sum();                  // 5.0.1 fix dodaje baze md5sum     
        $app_download='__DOWNLOAD__';     
        $app_updated='__UPDATED__';   
        $app_nothing2upgrade='__NOTHING2UPGRADE__';
        
        $channel=$this->getRequestParameter('channel');
        $package=$this->getRequestParameter('package');
        
        ob_start();
        
        $installer_web = new stInstallerFrontendWeb();     
        if ($all)    $installer_web->command('upgrade-all',array("$channel/$package"));         // upgrade all available packages
        else         $installer_web->command('upgrade',array("-fn","$channel/$package"));       // upgrade 1 package
                     
        $content = ob_get_clean();   
        $content = stPearFrontendWeb::getPearResult($content,'raw');

        // prese PEAR messages
        $messages=preg_split("/\n/",$content); $apps=array(); $result='';
        foreach ($messages as $line) 
        {                                                
             if (ereg("Starting to download",$line))  $apps[]=ereg_replace('Starting to download',$app_download,$line);
             if (ereg("upgrade ok",$line)) $result=$app_updated;
             if (ereg("Nothing to upgrade",$line)) $result=$app_nothing2upgrade;
        }                                             
        $this->apps=$apps; 
        $this->result=$result;      
    }       
                                    
    /** 
     * Upgrade all available packages.
     */
    public function executeUpgradeAll()
    {
        $this->executeUpgrade(true);        
    }
    
    /** 
     * Application list for synchronization.
     */
    public function executeSyncList()
    {   

        $regsync = new stRegisterSync();
        $apps_sync=$regsync->getAppsToSync();      
        $this->apps=$apps_sync;
        
        $file = sfConfig::get('sf_root_dir').DIRECTORY_SEPARATOR."install".DIRECTORY_SEPARATOR."db".DIRECTORY_SEPARATOR.".packages.reg";
        $this->linkToDownloadPackage = false;
        if (file_exists($file)) $this->linkToDownloadPackage = true;
    }     
    
    /** 
     * Reboot.
     *
     * @param   bool        $check              true - verify number packahes for upgrade, false - do not verify
     */
    public function executeReboot($check=true)
    {    
        // Zweryfikuj czas potrezbny do wykonania akcji
        // Pobierz ilosc aplikacji do instalacji                                           
        $regsync = new stRegisterSync();
        $apps_sync=$regsync->getAppsToSync();          
        $npkg=sizeof($apps_sync['all']);              
        if ((! $npkg>0) && ($check)) $this->redirect('installerweb');                                                 
         
        $time = new stServerExecutionTime();
        $time->setMax(($npkg*ST_MIN_EXECUTION_TIME_APP)+ST_MIN_EXECUTION_TIME);
        if ($time->check())
        {
            $this->time=true;
        } else {
            $this->time=false; 
            $this->time_server=$time->getServerTime();
            $this->time_min=$time->getMax();
        }                           

        $file = sfConfig::get('sf_root_dir').DIRECTORY_SEPARATOR."install".DIRECTORY_SEPARATOR."db".DIRECTORY_SEPARATOR.".packages.reg";
        $this->linkToDownloadPackage = false;
        if (file_exists($file)) $this->linkToDownloadPackage = true;
    }                                           
    
    /** 
     * Rescue.
     */
    public function executeRescue()
    {
        // force unlock downloads
        stStepsInstaller::unlockPackagesListFile();
        
        // $this->executeReboot(false);  
        // $this->setTemplate('reboot');
    }   
        
    /** 
     * Rescure Reboot.
     */
    public function executeRescueReboot()
    {
        $this->executeReboot(false);  
        $this->setTemplate('reboot');
    }    
    
    /** 
     * Synchronize downloaded applications.
     */
    public function executeSync()
    {   
        $regsync = new stRegisterSync();
        $apps_sync=$regsync->getAppsToSync();      
                               
        // application list for synchronization
        $this->apps=$apps_sync;

        $installer = new stInstaller('web');  
        $ui = new stInstallerOutputWeb();     
        $installer->setOutputObject($ui); 


        if (! empty($apps_sync['all'])) 
        {                        
                    
            // synchornize
            $installer->sync($apps_sync['all'],'Synchronization ('.sizeof($apps_sync['all']).')');                                                                         
        }
        else {
            // nothing to do
        }
    }        
    
    /** 
     * Code verification.
     * Find user unautorized modifications.
     */
    public function executeVerify()
    {
        $regsync = new stRegisterSync();
        $apps_sync=$regsync->getAppsToSync(); 
        
        $this->apps=array_merge($apps_sync['changed'],$apps_sync['added']);

        $npkg_all=sizeof($apps_sync['all']);        
        $npkg_changed=sizeof($this->apps);   
        $npkg_del=sizeof($apps_sync['deleted']);
        
        $this->npkg=$npkg_changed;
            
        if (! $npkg_all>0) {
           $this->redirect('installerweb');
        }                                   
        
        $file = sfConfig::get('sf_root_dir').DIRECTORY_SEPARATOR."install".DIRECTORY_SEPARATOR."db".DIRECTORY_SEPARATOR.".packages.reg";
        $this->linkToDownloadPackage = false;
        if (file_exists($file)) $this->linkToDownloadPackage = true;
    }
        
    /** 
     * Symfony Web task execution.
     */
    public function executeTask()
    {   
        $this->task=$this->getRequestParameter('task');  
        $this->content='';
        $this->error='';
        
        if (! empty($this->task))
        {
            $pakeweb = new stPakeWeb();
            $pakeweb->run($this->task);
            $this->content=$pakeweb->content;
            $this->error=$pakeweb->error;       
        }
       
    }    
    
    /** 
     * Symfony cc
     */
    public function executeCc()
    {   
        $pakeweb = new stPakeWeb();
        $pakeweb->run('cc');
        $this->content=$pakeweb->content;
        $this->error=$pakeweb->error;       
    }
    
    /** 
     * Upgrade provided Applications.
     */
    public function executeUpgradeBySteps()
    {
    	if (!$this->getRequestParameter('package') || !$this->getRequestParameter('version'))
    	{
    		$this->redirect('stInstallerWeb/upgradeList');
    	} else {
    	    $package = $this->getRequestParameter('package');
    	    $version = $this->getRequestParameter('version');
            stStepsInstaller::setUpgradeType('package', $package, $version);    	    
    	}
    }
    
    /** 
     * Upgrade all applications.
     */
    public function executeUpgradeAllBySteps()
    {                        
        stStepsInstaller::setUpgradeType('all');
    }
    
    public function executeListUpgrade()
    {
        ob_start();

        $installer_web = new stInstallerFrontendWeb();            
        $installer_web->command('list-upgrades');
        
        $this->packages = ob_get_clean();
        $this->packages = stPearFrontendWeb::getPearResult($content);
    } 
    
    /** 
     * Package upload.
     */
    public function executeUpload()
    {
        stRegisterSync::fixmd5sum(); 
        
        $file_name = $this->getRequest()->getFileName('file');          
        $file_dir=sfConfig::get('sf_root_dir').DIRECTORY_SEPARATOR.'install'.DIRECTORY_SEPARATOR.'cache';
        $file_path=$file_dir.DIRECTORY_SEPARATOR.$file_name;
        
        $this->getRequest()->moveFile('file', $file_path);
        
        $forced=$this->getRequestParameter('forced');
        $nodeps=$this->getRequestParameter('nodeps');
        
        if ($forced==1) $forced=true;
        else $forced=false;
        
        if ($nodeps==1) $nodeps=true;
        else $nodeps=false;
                                                                                    
        ob_start();
                                                                                                            
        $installer_web = new stInstallerFrontendWeb();            
        if ($forced) $command='force-install'; else $command='install';
        $installer_web->command($command,array("$file_path"),array('forced' => $forced, 'nodeps' => $nodeps));

        $content = ob_get_clean();
        $content=stPearFrontendWeb::getPearResult($content);

        // fix PEAR results
        if (! is_array($content))
        {
            if (ereg("\<div\>$",$content)) $content=preg_replace("/\<div\>$/","</div>",$content);     
        }

        $this->content=$content;    
    }
    
    /** 
     * Page not found message.
     */
    public function executePageNotFound()
    {
        
    }              
    
    /** 
     * Upgrade history.
     */
    public function executeHistory()
    {                            
        $history = new stInstallerHistory();
        $this->history_apps = array_reverse($history->getHistory());            
    }            
    
    /** 
     * RSS upgrade.
     */
    public function executeRss()
    {
        
    }     
    
    /** 
     * Expert tools.
     */
    public function executeTools()
    {
        // Check if optimization can be executed    
        $regsync = new stRegisterSync();
        $apps_sync=$regsync->getAppsToSync();      
        $this->optimization=true;
        if (! empty($apps_sync['all'])) $this->optimization=false; // there are apps in install/src 
        if (stLockUpdate::isLocked()) $this->optimization=false;   // install process is executed
    }        
        
    /** 
     * Verify all code. 
     */
    public function executeVerifyall()
    {
         $peari = new stPearInfo();
         $packages = $peari->getPackages();
         $this->count=sizeof($packages);  
    }      
       
    
    public function executeOptimization()
    {
        $this->executeCc();
    }
    
    /**
     * Symfony pear clear-cache
     */
    public function executePearcc()
    {
        ob_start();
        
        $installer_web = new stInstallerFrontendWeb();     
        $installer_web->command('clear-cache',array()); // pear clear-cache                     
        $this->content = ob_get_clean();   
    }
}    