<?php
/** 
 * SOTESHOP/stConfigPlugin
 *
 * Ten plik należy do aplikacji stConfigPlugin opartej na licencji (Open License SOTE) Otwarta Licencja SOTE.
 * Nie zmieniaj tego pliku, jeśli chcesz korzystać z automatycznych aktualizacji oprogramowania.
 * Jeśli chcesz wprowadzać swoje modyfikacje do programu, zapoznaj się z dokumentacją, jak zmieniać
 * oprogramowanie bez zmiany kodu bazowego http://www.sote.pl/modifications
 *
 * @package     stConfigPlugin
 * @subpackage  libs
 * @copyright   SOTE (www.sote.pl)
 * @license     http://www.sote.pl/license/open (Open License SOTE) Otwarta Licencja SOTE
 * @version     $Id: stConfig.class.php 5688 2010-06-22 10:23:21Z michal $
 * @author      Marcin Butlak <marcin.butlak@sote.pl>
 */

/** 
 * Klasa konfiguracji
 *
 * @package     stConfigPlugin
 * @subpackage  libs
 */
class stConfig
{
    /**
     * Wartosc typu Boolean
     */
    const BOOL = false;

    /**
     * Wartosc typu integer
     */
    const INT = 0;

    /**
     * Wartosc typu string
     */
    const STRING = '';

    protected $moduleName = '';
    protected $vars = array();
    protected $config = array();
    /**
     * Instancja obiektu sfContext
     * @var sfContext
     */
    protected $context = null;
    /**
     * Instanacja obiektu stBasket
     * @var stBasket object
     */
    protected static $instance = null;

    /**
     * Zwraca instancje obiektu
     * Przyklad:
     * stConfig::getInstance($this->getContext(), array('param1'=>stConfig::INT_TYPE, 'param2' => stConfig::BOOL_TYPE))
     *
     * @param   string      $context            instancja obiektu sfContext
     * @param   array       $vars               Lista parametrów zapisywanych do konfiguracji (deprecated)
     * @param   string      $module_name        Nazwa modulu, jesli nie podana przyjeta nazwa modulu w ktorym zostala zainicjalizowana instancja stConfig
     * @return   stConfig
     */
    public static function getInstance($context, $vars = null, $module_name = null)
    {
        // tymczasowy hack zapewniajacy kompatybilnosc wstecz
        if (is_string($vars))
        {
            $module_name = $vars;
            $vars = null;
        }

        if (is_null($module_name))
        {
            $module_name = $context->getModuleName();
        }

        if (!isset(self::$instance[$module_name]))
        {
            $class = __CLASS__;
            self::$instance[$module_name] = new $class();
        }

        self::$instance[$module_name]->initialize($context, $vars, $module_name);

        return self::$instance[$module_name];
    }

    /**
     * Inicjalizacja konfiguracji
     *
     * @param   string      $context            instancja obiektu sfContext
     * @param   array       $vars               Lista parametrów zapisywanych do konfiguracji
     * @param   string      $module_name        Nazwa modulu, jesli nie podana przyjeta nazwa modulu w ktorym zostala zainicjalizowana instancja stConfig
     */
    public function initialize($context, $vars = null, $module_name = null)
    {
        $this->context = $context;

        if ($vars)
        {
            $this->vars = $vars;
        }

        $this->moduleName = $module_name;

        $this->load();
    }

    /**
     * Zapisuje konfiguracje
     */
    public function save($force_cache_refresh = false)
    {
        $module_config_file = $this->getUserConfigPath();

        $config_data['all']['.auto_generated']['config']['last_modified'] = date('Y/m/d H:i:s');

        $config_data['all']['.auto_generated']['config']['fields'] = $this->config;

        $yaml = new sfYamlDumper();

        file_put_contents($module_config_file, $yaml->dump($config_data, 1000));

        if ($force_cache_refresh)
        {
            unlink($this->getCacheName($module_config_file));
        }
    }

    /**
     * Wczytuje konfiguracje i zwraca ja w postaci tablicy
     *
     * @deprecated beta-2
     * @return   array
     */
    public function load()
    {
        $user_config = $this->getUserConfigPath();

        $system_config = $this->getSystemConfigPath();

        if ($this->isEmpty())
        {
            $cache = $this->getCacheName($user_config);

            if (!is_readable($cache) || @filemtime($user_config) > filemtime($cache) || @filemtime($system_config) > filemtime($cache))
            {
                $config = $this->loadConfig();

                $this->writeCacheFile($user_config, $cache, '<?php $config = ' . var_export($config, true) . ' ?>');
            }
            else
            {
                include_once($cache);
            }


            $this->config = isset($config['all']) && is_array($config['all']['.auto_generated']['config']['fields']) ? $config['all']['.auto_generated']['config']['fields'] : array();

        }

        return $this->config;
    }

    /**
     * Brak parametrów konfiguracyjnych
     *
     * @return   bool
     */
    public function isEmpty()
    {
        return empty($this->config);
    }

    /**
     * Ustawia parametr w konfiguracji
     *
     * @param   string      $var                Nazwa parametru
     * @param   mixed       $value              Wartosc parametru
     * @return  bool        zwraca prawde jezeli parametr zostal ustawiony, w innym wypadku falsz
     */
    public function set($var, $value)
    {
        $this->config[$var] = $value;
    }

    /**
     * Pobiera parametr z konfiguracji
     *
     * @param   string      $var                Nazwa parametru
     * @param   const       $type               Zwraca wartość w określonym typie
     * @return  mixed       zwraca null w przypadku braku parametru
     */
    public function get($var, $type = false)
    {
        if (isset($this->config[$var]) && !empty($this->config[$var]))
        {
            return $this->config[$var];
        }

        if ($type === false && isset($this->config[$var]) && is_numeric($this->config[$var]))
        {
            $type = self::INT;
        }
        elseif ($type === false)
        {
            $type = self::STRING;
        }

        return $type;
    }

    /**
     * Nadpisuje konfigurację wartościami z żądania
     *
     * @param   string      $parameter          Nazwa parametru żądania zawierąjacego parametry do nadpisania
     */
    public function setFromRequest($parameter)
    {
        $request = $this->context->getRequest();

        $data = $request->getParameter($parameter, array());

        $vars = array_merge($this->config, $data);

        foreach ($vars as $key => $value)
        {
            $this->set($key, isset($data[$key]) ? $data[$key] : '');
        }
    }

    /**
     * Wczytuje konfigurację danego modułu/aplikacji, jeżeli taka istnieje i zwraca ja w postaci tablicy
     *
     * @deprecated beta-2
     * @param   string      $module_name        Nazwa modułu/aplikacji
     * @return  array       Tablica zmiennych konfiguracyjnych
     */
    public static function getConfig($module_name)
    {
        include (sfConfigCache::getInstance()->checkConfig(sfConfig::get('sf_data_dir') . DIRECTORY_SEPARATOR . 'config' . DIRECTORY_SEPARATOR . $module_name . '.yml'));

        /**
         * @todo byrzdki hack konieczny refactoring
         */
        $config = sfConfig::get('st_module_config');
        sfConfig::set('st_module_config', null);
        sfConfig::set($module_name, $config['fields']);

        return $config['fields'];
    }

    /**
     *
     * Zapisuje cache dla konfiguracji
     *
     * @param string $config Sciezka do pliku konfiguracyjnego
     * @param string $cache Sciezka do cache pliku konfiguracyjnego
     * @param string $data Dane do zapisania
     */
    protected function writeCacheFile($config, $cache, $data)
    {
        $fileCache = new sfFileCache(dirname($cache));
        $fileCache->initialize(array('lifeTime' => 86400 * 7));
        $fileCache->setWriteControl(false);
        $fileCache->setSuffix('');

        if (!$fileCache->set(basename($cache), '', $data))
        {
            throw new sfConfigurationException(sprintf('Unable to write config cache for "%s".', $config));
        }
    }

    /**
     *
     * Pobiera sciezke do pliku konfiguracyjnego w cache na podstawie sciezki do pliku orginalnego
     *
     * @param string $config Orginalna sciezka do pliku konfiguracyjnego
     * @return string Sciezka do pliku konfiguracyjnego w cache
     */
    protected function getCacheName($config)
    {
        $config  = str_replace(array('\\', '/', ' ', ':'), '_', $config).'.php';

        return sfConfig::get('sf_root_cache_dir') . DIRECTORY_SEPARATOR . 'st_config' . DIRECTORY_SEPARATOR . $config;
    }

    protected function getSystemConfigPath()
    {
       return sfConfig::get('sf_data_dir') . DIRECTORY_SEPARATOR . 'config' . DIRECTORY_SEPARATOR . $this->moduleName . '.yml';
    }

    protected function getUserConfigPath()
    {
        return sfConfig::get('sf_data_dir') . DIRECTORY_SEPARATOR . 'config' . DIRECTORY_SEPARATOR . '__' . $this->moduleName . '.yml';
    }

    protected function loadConfig()
    {
        $config = array();

        require_once (sfConfig::get('sf_plugins_dir').DIRECTORY_SEPARATOR.'stInstallerPlugin'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'helper'.DIRECTORY_SEPARATOR.'stArrayHelper.php');

        $yaml = new sfYamlParser();

        $system_config = $this->getSystemConfigPath();

        $user_config = $this->getUserConfigPath();

        if (is_file($system_config))
        {
            $config = $yaml->parse(file_get_contents($system_config));
        }

        if (is_file($user_config))
        {
            $config = st_array_merge_recursive3($config, $yaml->parse(file_get_contents($user_config)));
        }

        return $config;
    }
}